//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __DetConst__
#define __DetConst__

using namespace std;

static const int N_KNOWN_DET = 4; // number of known LV detectors

// Detector's indexes
static const int DET_INDEX[N_KNOWN_DET] = {0,  // Virgo
					   1,  // Hanford 1
					   2,  // Hanford 2
					   3}; // Livingston

// Detector's names
static const string DET_NAME[N_KNOWN_DET] = {"Virgo", 
					     "Hanford1", 
					     "Hanford2", 
					     "Livingston"};

// Detector's prefixes
static const string DET_PREFIX[N_KNOWN_DET] = {"V1", 
					       "H1", 
					       "H2", 
					       "L1"};

// x-component of vertex location in Earth-centered frame (m)
static const double DET_VERTEX_LOCATION_X_SI[N_KNOWN_DET] = {4.54637409900e+06,
							     -2.16141492636e+06, 
							     -2.16141492636e+06, 
							     -7.42760447238e+04};

// y-component of vertex location in Earth-centered frame (m)
static const double DET_VERTEX_LOCATION_Y_SI[N_KNOWN_DET] = {8.42989697626e+05,
							     -3.83469517889e+06, 
							     -3.83469517889e+06, 
							     -5.49628371971e+06};

// z-component of vertex location in Earth-centered frame (m)
static const double DET_VERTEX_LOCATION_Z_SI[N_KNOWN_DET] = {4.37857696241e+06,
							     4.60035022664e+06, 
							     4.60035022664e+06, 
							     3.22425701744e+06};

// x-component of unit vector pointing along x arm in Earth-centered frame
static const double DET_ARM_X_DIRECTION_X[N_KNOWN_DET] = {-0.70045821479, 
							  -0.22389266154, 
							  -0.22389266154, 
							  -0.95457412153};

// y-component of unit vector pointing along x arm in Earth-centered frame
static const double DET_ARM_X_DIRECTION_Y[N_KNOWN_DET] = {0.20848948619, 
							  0.79983062746, 
							  0.79983062746, 
							  -0.14158077340};

// z-component of unit vector pointing along x arm in Earth-centered frame
static const double DET_ARM_X_DIRECTION_Z[N_KNOWN_DET] = {0.68256166277, 
							  0.55690487831, 
							  0.55690487831, 
							  -0.26218911324};

// x-component of unit vector pointing along y arm in Earth-centered frame
static const double DET_ARM_Y_DIRECTION_X[N_KNOWN_DET] = {-0.05379255368, 
							  -0.91397818574, 
							  -0.91397818574, 
							  0.29774156894};

// y-component of unit vector pointing along y arm in Earth-centered frame
static const double DET_ARM_Y_DIRECTION_Y[N_KNOWN_DET] = {-0.96908180549, 
							  0.02609403989, 
							  0.02609403989, 
							  -0.48791033647};

// z-component of unit vector pointing along y arm in Earth-centered frame
static const double DET_ARM_Y_DIRECTION_Z[N_KNOWN_DET] = {0.24080451708, 
							  -0.40492342125, 
							  -0.40492342125, 
							  -0.82054461286};

// maximum light travel time between detectors [s]
static const double NET_LIGHTTRAVELTIMEMAX[N_KNOWN_DET][N_KNOWN_DET] = {
  { 0.000, 0.025, 0.025, 0.027 },
  { 0.025, 0.000, 0.000, 0.002 },
  { 0.025, 0.000, 0.000, 0.002 },
  { 0.027, 0.002, 0.002, 0.000 }};

/**
 * Returns the LV detector index associated to a detector prefix.
 * It returns -1 if no detector matches the given prefix.
 * @param aDetPrefix detector prefix
 */
inline int GetLVDetIndex(const string aDetPrefix){
  for(int d=0; d<N_KNOWN_DET; d++)
    if(!aDetPrefix.compare(DET_PREFIX[d])) return d;
  return -1;
}


#endif
